/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once

#include <cassert>
#include <cstring>
#include <memory>
#include <string>
#include <vector>
#include <iostream>
#include "parser/parser.h"
#include "system/sm.h"
#include "common/common.h"

class Query{
    public:
    std::shared_ptr<ast::TreeNode> parse;
    // where条件
    std::vector<Condition> conds;
    //having条件
    std::vector<AgreCondition> having_conds;
    // 投影列
    std::vector<TabCol> cols;//假定在聚合函数中，TablCol是第一个值
    std::vector<AgreCol> agre_cols;
    // 表名
    std::vector<std::string> tables;
    // update 的set 值
    std::vector<SetClause> set_clauses;
    //insert 的values值
    std::vector<Value> values;
    //groupby的列
    std::vector<TabCol> groupby_cols;

    bool has_sublink = false;
    Query(){}

    std::vector<TabCol> convert_agrecol_to_TabCol(){
        std::vector<TabCol> tmp;
        for(auto &item : agre_cols){
            tmp.push_back(item.alias);
        }
        return tmp;
    }
};

class Analyze
{
private:
    SmManager *sm_manager_;
public:
    Analyze(SmManager *sm_manager) : sm_manager_(sm_manager){}
    ~Analyze(){}

    std::shared_ptr<Query> do_analyze(std::shared_ptr<ast::TreeNode> root);

private:
    void get_sel_col(const std::vector<std::shared_ptr<ast::Col>> &sv_cols,
                     std::vector<TabCol> &regular_col,
                     std::vector<AgreCol> &agre_col,
                     const std::vector<ColMeta> &all_cols);
    void get_all_cols(const std::vector<std::string> &tab_names, std::vector<ColMeta> &all_cols);
    void get_clause(const std::vector<std::shared_ptr<ast::BinaryExpr>> &sv_conds, std::vector<Condition> &conds,bool &has_sublink);
    void get_groupby_col(const std::vector<std::shared_ptr<ast::Col>>&sv_groupyby_cols,std::vector<TabCol>& groupby_cols );
    void get_having_clause(const std::vector<std::shared_ptr<ast::BinaryExpr>> &sv_having_conds,std::vector<AgreCondition> &having_conds);

    //void get_const_values(Condition &cond,std::vector<std::shared_ptr<Value>> values);

    TabCol check_column(const std::vector<ColMeta> &all_cols, TabCol target);
    void check_clause(const std::vector<std::string> &tab_names, std::vector<Condition> &conds);
    void check_groupby_clause(const std::vector<TabCol> &groupby_cols,std::vector<TabCol> regular_cols );
    void check_having_clause(const std::vector<std::string> &tab_names ,std::vector<AgreCondition> &having_conds);
    Value convert_sv_value(const std::shared_ptr<ast::Value> &sv_val);
    CompOp convert_sv_comp_op(ast::SvCompOp op);
    AgreType convert_sv_agretype(ast::SvAgreType type);
    UpdateOp convert_sv_update_op(ast::SvUpdateOp op);
};

