/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#include "rm_scan.h"
#include "rm_file_handle.h"

/**
 * @brief 初始化file_handle和rid
 * @param file_handle
 */
RmScan::RmScan(const RmFileHandle *file_handle) : file_handle_(file_handle), page_buffer(&(file_handle_->file_hdr_),(Page*) nullptr) {
    // Todo:
    // 初始化file_handle和rid（指向第一个存放了记录的位置）
    rid_.page_no = 1;
    rid_.slot_no = -1;
    //next();
    auto max_rec_num = file_handle_->file_hdr_.num_records_per_page;
    auto num_page = file_handle_->file_hdr_.num_pages;
    for (; rid_.page_no < num_page; rid_.page_no++) {
        page_buffer = file_handle_->fetch_page_handle(rid_.page_no);
        int next_slot = Bitmap::next_bit(true, page_buffer.bitmap, max_rec_num, -1);
        if (next_slot < max_rec_num) {
            rid_.slot_no = next_slot;
            //file_handle_->buffer_pool_manager_->unpin_page({file_handle_->fd_,rid_.page_no}, false);
            return;
        }
        file_handle_->buffer_pool_manager_->unpin_page({file_handle_->fd_,rid_.page_no}, false);
        // 没有空的slot
        //rid_.slot_no = -1;
    }
}

/**
 * @brief 找到文件中下一个存放了记录的位置
 */
void RmScan::next() {
    // Todo:
    // 找到文件中下一个存放了记录的非空闲位置，用rid_来指向这个位置
    auto max_rec_per_page = file_handle_->file_hdr_.num_records_per_page;
    auto num_page = file_handle_->file_hdr_.num_pages;
    while(rid_.page_no < num_page){
        int next_slot = Bitmap::next_bit(true, page_buffer.bitmap, max_rec_per_page, rid_.slot_no);
        if(next_slot < max_rec_per_page){
            rid_.slot_no = next_slot;
            return;
        }
        //更新buffer_page
        file_handle_->buffer_pool_manager_->unpin_page({file_handle_->fd_,rid_.page_no},false);
        rid_.page_no++;
        if(rid_.page_no < num_page){
            page_buffer = file_handle_->fetch_page_handle(rid_.page_no);
        }
        rid_.slot_no = -1;
    }
//    for (; rid_.page_no < file_handle_->file_hdr_.num_pages; rid_.page_no++) {
//        //RmPageHandle page_handler = file_handle_->fetch_page_handle(rid_.page_no);//TODO 存在先写：对同一页面频繁获取
//        int next_slot = Bitmap::next_bit(true, page_buffer.bitmap, max_rec_per_page, rid_.slot_no);
//        if (next_slot < max_rec_per_page) {
//            rid_.slot_no = next_slot;
//            //file_handle_->buffer_pool_manager_->unpin_page({file_handle_->fd_,rid_.page_no}, false);//TODO 频繁unpin
//            return;
//        }
//        //file_handle_->buffer_pool_manager_->unpin_page({file_handle_->fd_,rid_.page_no}, false);
//        // 没有空的slot
//        rid_.slot_no = -1;
//    }
}

/**
 * @brief ​ 判断是否到达文件末尾
 */
bool RmScan::is_end() const {
    // Todo: 修改返回值
    return rid_.page_no == INVALID_PAGE_ID || rid_.slot_no < 0;
}

/**
 * @brief RmScan内部存放的rid
 */
Rid RmScan::rid() const {
    return rid_;
}